import time

import click
import loguru

from docx_translate import version_info
from docx_translate.core.translate import DocxTranslator


CONTEXT_SETTINGS = dict(help_option_names=['-?', '-h', '--help'])

epilog = click.style('''
\n\b
example:
    {prog} input.docx -o output.docx
    {prog} input.docx -o output.docx -t zh-CN
                     
contact: {author}<{author_email}>                   
''')

@click.command(
    name=version_info['prog'],
    help=click.style(version_info['desc'], italic=True, fg='cyan', bold=True),
    context_settings=CONTEXT_SETTINGS,
    no_args_is_help=True,
)
@click.version_option(version=version_info['version'], prog_name=version_info['prog'])
@click.argument('input_file', type=click.Path(exists=True))
@click.option('-o', '--output', type=click.Path(), help='output file', default='output.docx', show_default=True)
@click.option('-t', '--target', help='target language', default='en', show_default=True)
def cli(input_file, output, target):
    start_time = time.time()
    trans = DocxTranslator(target=target)
    trans.translate_docx(input_file, output)
    loguru.logger.info(f'time elapsed: {time.time() - start_time:.2f}s')


def main():
    cli()


if __name__ == '__main__':
    main()



from abc import ABC, abstractmethod


class BaseTranslator(ABC):
    """
    Base class for Translator, define the unified interface for translation
    """
    
    def __init__(self, source: str = 'auto', target: str = 'en'):
        """
        :param source: source language code, default is 'auto'
        :param target: target language code, default is 'en'
        """
        self.source = source
        self.target = target

    @abstractmethod
    def translate(self, text: str) -> str:
        """
        :param text: text to be translated
        :return: translated text
        """
        raise NotImplementedError



import loguru
from docx import Document

from .translator import GoogleTranslator


class DocxTranslator:
    def __init__(self, source: str = 'auto', target: str = 'en'):
        """
        :param source: source language code, default is "auto"
        :param target: target language code, default is "en"
        """
        self.logger = loguru.logger
        self.translator = GoogleTranslator(source=source, target=target)

    def translate_docx(self, input_path: str, output_path: str):
        """
        :param input_path: input file path
        :param output_path: output file path
        """
        doc = Document(input_path)

        # translate header
        for section in doc.sections:
            if section.header.is_linked_to_previous:
                continue
            for paragraph in section.header.paragraphs:
                if paragraph.text.strip():
                    self.logger.debug(f'>>> translating header: {paragraph.text}')
                    translated_text = self.translator.translate(paragraph.text)
                    paragraph.text = translated_text

        # translate footer
        for section in doc.sections:
            if section.footer.is_linked_to_previous:
                continue
            for paragraph in section.footer.paragraphs:
                if paragraph.text.strip():
                    self.logger.debug(f'>>> translating footer: {paragraph.text}')
                    translated_text = self.translator.translate(paragraph.text)
                    paragraph.text = translated_text

        # translate paragraphs
        for paragraph in doc.paragraphs:
            if paragraph.text.strip():
                self.logger.debug(f'>>> translating paragraph: {paragraph.text}')
                translated_text = self.translator.translate(paragraph.text)
                paragraph.text = translated_text

        # translate tables
        for n, table in enumerate(doc.tables, 1):
            self.logger.debug(f'>>> translating table {n}')
            for row in table.rows:
                for cell in row.cells:
                    if cell.text.strip():
                        translated_text = self.translator.translate(cell.text)
                        cell.text = translated_text

        # save translated docx
        doc.save(output_path)
        self.logger.info(f'>>> translated docx saved to {output_path}')


from googletranslatepy import Translator

from .base import BaseTranslator


class GoogleTranslator(BaseTranslator):
    """Google Translator with `googletranslatepy`
    """
    
    def __init__(self, source: str = 'auto', target: str = 'en'):
        super().__init__(source, target)
        self.translator = Translator(source=source, target=target)

    def translate(self, text: str) -> str:
        """
        :param text: text to be translated
        :return: translated text
        """
        return self.translator.translate(text)
